{+--------------------------------------------------------------------------+
 | Class:       Application  ( Tree Space )
 | Created:     1.22.99
 | Author:      William Miller
 | Copyright    1999, all rights reserved.
 | Description: A simple application, to display disk size and free space
 |              in a TreeView - works with drives > 2 gig
 |              The original is in 'Delphi 4 '.
 | Version:     1.0
 | Status:      FreeWare
 | It's provided as is, without a warranty of any kind.
 | You use it at your own risk.
 | E-Mail me at w2m@netheaven.com
 +--------------------------------------------------------------------------+}

unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, Free, SystemImageList, ImgList, ToolWin, DFSToolBar,
  Menus, ItemProp, DFSStatusBar, Placemnt, xProcs, TrayIcon, RXShell,
  AppEvent;

type
  TForm1 = class(TForm)
    TreeView1: TTreeView;
    DFSToolBar1: TDFSToolBar;
    ToolButton1: TToolButton;
    ColorImageList: TImageList;
    SystemImageList1: TdfsSystemImageList;
    ToolButton2: TToolButton;
    PopupMenu1: TPopupMenu;
    Properties1: TMenuItem;
    N1: TMenuItem;
    Refresh1: TMenuItem;
    Exit1: TMenuItem;
    ToolButton3: TToolButton;
    ToolButton4: TToolButton;
    GSImageList: TImageList;
    ToolButton5: TToolButton;
    ToolButton6: TToolButton;
    ToolButton7: TToolButton;
    DFSStatusBar1: TDFSStatusBar;
    FormStorage1: TFormStorage;
    Explorer1: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure ExitClick(Sender: TObject);
    procedure RefreshClick(Sender: TObject);
    procedure PropertiesClick(Sender: TObject);
    procedure ExpandClick(Sender: TObject);
    procedure ExplorerClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure TreeView1Change(Sender: TObject; Node: TTreeNode);
    procedure kbTrayIcon1DoubleClick(Sender: TObject; Button: TMouseButton;
      var DefaultAction: Boolean);
    procedure Exit2Click(Sender: TObject);
    procedure Properties2Click(Sender: TObject);
    procedure AppEventsActivate(Sender: TObject);
    procedure AppEventsMinimize(Sender: TObject);
    procedure AppEventsRestore(Sender: TObject);
    procedure TrayIconDblClick(Sender: TObject);
  private
    { Private declarations }
    function DiskInDrive(const Drive: char): Boolean;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

uses Main;

{$R *.DFM}

// is disk in drive?
function TForm1.DiskInDrive(const Drive: char): Boolean;
var
  DrvNum: byte;
  EMode: Word;
  Drev: Char;
  MyTreeNode1: TTreeNode;
begin
  Result := False;
  Drev := AnsiUpperCase(Drive)[1];
  if not (Drev in ['A'..'Z']) then
    Exit;
  DrvNum := ord(Drev) - $40; {1-26}
  EMode := SetErrorMode(SEM_FAILCRITICALERRORS);
  try
    if DiskSize(DrvNum) <> -1 then begin
      Result := True;
    end;
  finally
    SetErrorMode(EMode);
  end;
end;

// get drive info and fill treelist
procedure TForm1.FormCreate(Sender: TObject);
var
  MyTreeNode1: TTreeNode;
  i: integer;
  Drive: String;
  TotalSpace: Real;
  FreeSpace: Real;
  UsedSpace: Real;
  TS: string;
  FS: string;
  US: string;
  Attrs: TSystemFileAttributes;
begin

  with TreeView1.Items do
  begin
    // remove any existing nodes
    Clear;
    // Check for valid drives
    for i := 65 to 89 do
      if DiskInDrive(Char(I)) then
      begin
        // Add a drive
        MyTreeNode1 := Add(nil, Char(I)+':\');
        Attrs := [sfaDirectory]; // tell it we want the folder image index.
        MyTreeNode1.ImageIndex := SystemImageList1.GetImageIndex(Char(I)+':\', True, Attrs);
        MyTreeNode1.SelectedIndex := SystemImageList1.GetImageIndex(Char(I)+':\', True, Attrs);
        // Get total and free space
        Drive := Char(I)+':\';
        Totalspace := 0;
        FreeSpace := 0;
        UsedSpace := 0;
        GetDiskFreeSpaceBytes(Drive, TotalSpace, FreeSpace);
        UsedSpace := TotalSpace - FreeSpace;
        //Add a child node to the node just added
        AddChild(MyTreeNode1,'Capacity ' + strByteSizeReal(TotalSpace));  { Add a drive size }
        MyTreeNode1.getFirstChild.ImageIndex := 12;
        MyTreeNode1.getFirstChild.SelectedIndex := 12;
        // Add a child node to the node just added
        AddChild(MyTreeNode1,'Used ' + strByteSizeReal(UsedSpace));    { Add a used space }
        MyTreeNode1.GetLastChild.ImageIndex := 12;
        MyTreeNode1.GetLastChild.SelectedIndex := 12;
        // Add a child node to the node just added
        AddChild(MyTreeNode1,'Free ' + strByteSizeReal(FreeSpace));    { Add a free space }
        MyTreeNode1.GetLastChild.ImageIndex := 12;
        MyTreeNode1.GetLastChild.SelectedIndex := 12;
      end;
    end;
end;

procedure TForm1.ExitClick(Sender: TObject);
begin
  Close;
end;

// Reset the treelist
procedure TForm1.RefreshClick(Sender: TObject);
begin
  TreeView1.Items.Clear;
  FormCreate(Self);
end;

procedure TForm1.PropertiesClick(Sender: TObject);
begin
  if not DisplayPropertiesDialog(TreeView1.Selected.Text, Handle) then
    ShowMessage('There are no properties for this item!');
end;

procedure TForm1.ExpandClick(Sender: TObject);
begin
    If ToolButton4.Down then
      TreeView1.FullExpand
    else
      TreeView1.FullCollapse;
end;

procedure TForm1.ExplorerClick(Sender: TObject);
begin
  DFSExplorerForm := TDFSExplorerForm.Create(Application);
  DFSExplorerForm.Show;
  DFSExplorerForm.Tree.SetFocus;
end;

procedure TForm1.FormShow(Sender: TObject);
begin
 If ToolButton4.Down then
      TreeView1.FullExpand
    else
      TreeView1.FullCollapse;

    TreeView1.Selected := TreeView1.TopItem;
end;

procedure TForm1.TreeView1Change(Sender: TObject; Node: TTreeNode);
begin
  DFSStatusBar1.SimpleText := Node.Text;
end;

procedure TForm1.kbTrayIcon1DoubleClick(Sender: TObject;
  Button: TMouseButton; var DefaultAction: Boolean);
begin
  Show;
end;

procedure TForm1.Exit2Click(Sender: TObject);
begin
  Close;
end;

procedure TForm1.Properties2Click(Sender: TObject);
begin
  Application.Restore;
  Application.BringToFront;
end;

procedure TForm1.AppEventsActivate(Sender: TObject);
begin
  if Screen.ActiveForm <> nil then Screen.ActiveForm.BringToFront;
end;

procedure TForm1.AppEventsMinimize(Sender: TObject);
begin
  if NewStyleControls then ShowWindow(Application.Handle, SW_HIDE);
end;

procedure TForm1.AppEventsRestore(Sender: TObject);
begin
  if NewStyleControls then ShowWindow(Application.Handle, SW_SHOW);
end;

procedure TForm1.TrayIconDblClick(Sender: TObject);
begin
  Application.Restore;
  Application.BringToFront;
end;

end.
