unit Diskspace;

interface

uses Classes, Windows;

type
  TGetDiskFreeSpaceEx = function (lpDirectoryName: PChar;
  var lpFreeBytesAvailableToCaller, lpTotalNumberOfBytes, lpTotalNumberOfFreeBytes: Comp): BOOL; stdcall;

procedure GetDiskSpaceKB(const Drive: String; var TotalKBytes, FreeKBytes: DWord);
procedure GetDiskSpaceBytes(const Drive: String; var TotalKBytes, FreeKBytes: Double);

///////////////////////////////////////////////////////////////////////////////

implementation
uses SysUtils, ShellApi, Forms, Dialogs, Messages;

var FGetDiskFreeSpaceEx: TGetDiskFreeSpaceEx;

// Function to retrieve disk size and free space on a drive
procedure GetDiskSpaceKB(const Drive: String; var TotalKBytes, FreeKBytes: DWord);
var SectorsPerCluster,
    BytesPerSector,
    NumberOfFreeClusters,
    TotalNumberOfClusters : DWord;
    TotalNumberOfBytes,
    FreeBytesAvailableToCaller,
    TotalNumberOfFreeBytes: Comp;
begin
  TotalKBytes := 0;
  FreeKBytes  := 0;
  if Assigned(FGetDiskFreeSpaceEx) then begin
    if FGetDiskFreeSpaceEx(PChar(Drive), FreeBytesAvailableToCaller, TotalNumberOfBytes, TotalNumberOfFreeBytes) then begin
      TotalKBytes := Round(TotalNumberOfBytes/1024);
      FreeKBytes  := Round(TotalNumberOfFreeBytes/1024);
    end;
  end else
    if GetDiskFreeSpace(PChar(Drive), SectorsPerCluster, BytesPerSector, NumberOfFreeClusters, TotalNumberOfClusters) then begin
      TotalKBytes := Round(SectorsPerCluster * BytesPerSector / 1024 * TotalNumberOfClusters);
      FreeKBytes  := Round(SectorsPerCluster * BytesPerSector / 1024 * NumberOfFreeClusters);
    end;
end;

// Function to retrieve the free space on a drive
procedure GetDiskSpaceBytes(const Drive: String; var TotalKBytes, FreeKBytes: Double);
var SectorsPerCluster,
    BytesPerSector,
    NumberOfFreeClusters,
    TotalNumberOfClusters : DWord;
    TotalNumberOfBytes,
    FreeBytesAvailableToCaller,
    TotalNumberOfFreeBytes: Comp;
begin
  TotalKBytes := 0;
  FreeKBytes  := 0;
  if Assigned(FGetDiskFreeSpaceEx) then begin
    if FGetDiskFreeSpaceEx(PChar(Drive), FreeBytesAvailableToCaller, TotalNumberOfBytes, TotalNumberOfFreeBytes) then begin
      TotalKBytes := TotalNumberOfBytes;
      FreeKBytes  := TotalNumberOfFreeBytes;
    end;
  end else
    if GetDiskFreeSpace(PChar(Drive), SectorsPerCluster, BytesPerSector, NumberOfFreeClusters, TotalNumberOfClusters) then begin
      TotalKBytes := SectorsPerCluster * BytesPerSector * TotalNumberOfClusters;
      FreeKBytes  := SectorsPerCluster * BytesPerSector * NumberOfFreeClusters;
    end;
end;


var h: THandle;

// Main Initialization of this unit
initialization

  // Get GetDiskFreeSpaceEx function
  h := LoadLibrary('Kernel32');
  if h<>0 then begin
    FGetDiskFreeSpaceEx := GetProcAddress(h, 'GetDiskFreeSpaceExA');
    FreeLibrary(h);
  end;

finalization

end.
