{$I DFS.INC}
unit main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, ComCtrls, SystemTreeView, ExtCtrls, SystemControlPack,
  SystemListView;

type
  TSimpleExplorerForm = class(TForm)
    MainMenu: TMainMenu;
    StatusBar: TStatusBar;
    Tree: TdfsSystemTreeView;
    Splitter1: TSplitter;
    List: TdfsSystemListView;
    File1: TMenuItem;
    ExitItem: TMenuItem;
    N1: TMenuItem;
    NewFolderItem: TMenuItem;
    Edit1: TMenuItem;
    N4: TMenuItem;
    PasteItem: TMenuItem;
    CopyItem: TMenuItem;
    CutItem: TMenuItem;
    View1: TMenuItem;
    DeleteItem: TMenuItem;
    RenameItem: TMenuItem;
    PropertiesItem: TMenuItem;
    SelectAllItem: TMenuItem;
    LargeIconsItem: TMenuItem;
    SmallIconsItem: TMenuItem;
    ListItem: TMenuItem;
    DetailsItem: TMenuItem;
    N2: TMenuItem;
    RefreshItem: TMenuItem;
    procedure TreeChange(Sender: TObject; Node: TTreeNode);
    procedure ListChange(Sender: TObject; Item: TListItem;
      Change: TItemChange);
    procedure NewFolderItemClick(Sender: TObject);
    procedure RenameItemClick(Sender: TObject);
    procedure ExitItemClick(Sender: TObject);
    procedure MultiListItemsFunctionClick(Sender: TObject);
    procedure SelectAllItemClick(Sender: TObject);
    procedure LargeIconsItemClick(Sender: TObject);
    procedure SmallIconsItemClick(Sender: TObject);
    procedure ListItemClick(Sender: TObject);
    procedure DetailsItemClick(Sender: TObject);
    procedure RefreshItemClick(Sender: TObject);
    procedure TreeOrListEnter(Sender: TObject);
    procedure ListPopulated(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    procedure UpdateUI;
    { Private declarations }
  public
    { Public declarations }
  end;

var
  SimpleExplorerForm: TSimpleExplorerForm;

implementation

uses
   CommCtrl, ItemProp, ShlObj;

{$R *.DFM}

procedure TSimpleExplorerForm.UpdateUI;
var
  b: boolean;
  ND: TFolderItemData;
begin
  ND := Tree.NodeData[Tree.Selected];
  if ND <> NIL then
  begin
    if ((ND.Attributes and SFGAO_FILESYSTEM) <> 0) and
       ((ND.Attributes and SFGAO_FILESYSANCESTOR) = 0) then
      Caption := 'Exploring - ' + Tree.Directory
    else
      Caption := 'Exploring - ' + Tree.Selected.Text;
  end else
    Caption := 'Exploring - Neverneverland';

  RenameItem.Enabled := (Tree.Focused and (Tree.Selected <> NIL)) or
     (List.Focused and (List.SelCount = 1));
  b := (Tree.Focused and (Tree.Selected <> NIL)) or
     (List.Focused and (List.Selected <> NIL));
  DeleteItem.Enabled := b;
  PropertiesItem.Enabled := b;
  CutItem.Enabled := b;
  CopyItem.Enabled := b;
end;

procedure TSimpleExplorerForm.TreeChange(Sender: TObject; Node: TTreeNode);
begin
  UpdateUI;
end;

procedure TSimpleExplorerForm.ListChange(Sender: TObject; Item: TListItem;
  Change: TItemChange);
begin
  UpdateUI;
end;

procedure TSimpleExplorerForm.NewFolderItemClick(Sender: TObject);
begin
  if Tree.AddNewNode(Tree.Selected, 'New Folder', TRUE) then
    Tree.Selected.EditText
  else
    MessageDlg('Could not add folder.', mtError, [mbOk], 0);
 end;

procedure TSimpleExplorerForm.RenameItemClick(Sender: TObject);
begin
 if Tree.Focused and (Tree.Selected <> NIL) then
   Tree.Selected.EditText
 else if List.Focused and (List.Selected <> NIL) then
   List.Selected.EditCaption;
end;

procedure TSimpleExplorerForm.ExitItemClick(Sender: TObject);
begin
  Close;
end;

// Event handler for all menu items that need to support multiple items selected
// in the list view.
procedure TSimpleExplorerForm.MultiListItemsFunctionClick(Sender: TObject);
var
  Data: TFolderItemData;
  Verb: string;
  SelData: PPIDLArray;
  SelCount: integer;
begin
  SelData := NIL;
  SelCount := 0;
  if Sender = PasteItem then
  begin
    Verb := 'paste';
    Data := Tree.NodeData[Tree.Selected];
  end else begin
    if Sender = CutItem then
      Verb := 'cut'
    else if Sender = CopyItem then
      Verb := 'copy'
    else if Sender = DeleteItem then
      Verb := 'delete'
    else if Sender = PropertiesItem then
      Verb := 'properties';

    if Tree.Focused and (Tree.Selected <> NIL) then
      Data := Tree.NodeData[Tree.Selected]
    else if List.Focused and (List.Selected <> NIL) then
    begin
      Data := List.ItemData[List.Selected];
      if List.SelCount > 1 then
        SelCount := List.GetSelectedPIDLs(SelData);
    end else
      Data := NIL;
  end;

  if Data <> NIL then
  begin
    try
      if (SelCount > 0) and (SelData <> NIL) then
{$IFDEF DFS_COMPILER_4_UP}
        ItemProp.PerformVerb(Verb, Data.SFParent, SelData^[0], Data.Attributes,
           Handle, SelCount)
      else
        ItemProp.PerformVerb(Verb, Data.SFParent, Data.FIDList, Data.Attributes,
           Handle, 1);
{$ELSE}
        ItemProp.PerformVerbPIDL(Verb, Data.SFParent, SelData^[0],
           Data.Attributes, Handle, SelCount)
      else
        ItemProp.PerformVerbPIDL(Verb, Data.SFParent, Data.FIDList,
           Data.Attributes, Handle, 1);
{$ENDIF}
    finally
      // GetSelectedPIDLs allocates memory, we must free it.
      FreeMem(SelData);
    end;
  end;
end;

procedure TSimpleExplorerForm.SelectAllItemClick(Sender: TObject);
begin
  // Why isn't there a TListView.SelectAll or TListItems.SelectAll to do this?
  ListView_SetItemState(List.Handle, -1, LVIS_SELECTED, LVIS_SELECTED);
end;

procedure TSimpleExplorerForm.LargeIconsItemClick(Sender: TObject);
begin
  List.ViewStyle := vsIcon;
end;

procedure TSimpleExplorerForm.SmallIconsItemClick(Sender: TObject);
begin
  List.ViewStyle := vsSmallIcon;
end;

procedure TSimpleExplorerForm.ListItemClick(Sender: TObject);
begin
  List.ViewStyle := vsList;
end;

procedure TSimpleExplorerForm.DetailsItemClick(Sender: TObject);
begin
  List.ViewStyle := vsReport;
end;

procedure TSimpleExplorerForm.RefreshItemClick(Sender: TObject);
begin
// store selection
  Tree.Reset;
  List.Reset;
// reset selection
end;

procedure TSimpleExplorerForm.TreeOrListEnter(Sender: TObject);
begin
  UpdateUI;
end;

procedure TSimpleExplorerForm.ListPopulated(Sender: TObject);
var
  x: integer;
  s: string;
  {$IFDEF DFS_COMPILER_4_UP}
  FreeSpace: Int64;
  SpaceUsed: Int64;
  {$ELSE}
  FreeSpace: comp;
  SpaceUsed: comp;
  {$ENDIF}
begin
  if Tree.Focused and (Tree.Selected <> NIL) then
  begin
    StatusBar.Panels[0].Text := IntToStr(List.Items.Count) + ' object(s)';
    if (Tree.NodeData[Tree.Selected].Attributes and SFGAO_FILESYSTEM) <> 0 then
    begin
      // Space used by all items and disk free space.
      SpaceUsed := 0;
      for x := 0 to List.Items.Count-1 do
        SpaceUsed := SpaceUsed + List.ItemData[List.Items[x]].FileSize;
      if SpaceUsed < 1024 then
        s := Format('%f bytes', [SpaceUsed / 1])
      else if SpaceUsed < 10485756 then
        s := Format('%fKB', [(SpaceUsed) / 1024])
      else if SpaceUsed < 1073741824 then
        s := Format('%fMB', [(SpaceUsed) / 1048576])
      else
        s := Format('%fGB', [(SpaceUsed) / 1073741824]);

      FreeSpace := DiskFree(Ord(UpCase(Tree.Selection[1])) - 64);
      s := s + ' (Disk free space: ';
      if FreeSpace < 1024 then
        s := s + Format('%f bytes', [FreeSpace / 1])
      else if FreeSpace < 10485756 then
        s := s + Format('%fKB', [(FreeSpace) / 1024])
      else if FreeSpace < 1073741824 then
        s := s + Format('%fMB', [(FreeSpace) / 1048576])
      else
        s := s + Format('%fGB', [(FreeSpace) / 1073741824]);
        
      s := s + ')';
    end else
      s := '';
    StatusBar.Panels[1].Text := s;
  end else if List.Focused and (List.Selected <> NIL) then
  begin
    StatusBar.Panels[0].Text := IntToStr(List.SelCount) + ' object(s) ' +
       'selected';
    if (List.ItemData[List.Selected].Attributes and SFGAO_FILESYSTEM) <> 0 then
    begin
      // Size of selected objects, blank if 0
      StatusBar.Panels[1].Text := 'something';
    end else
      StatusBar.Panels[1].Text := '';
  end;
end;

procedure TSimpleExplorerForm.FormCreate(Sender: TObject);
begin
{$IFDEF DFS_COMPILER_4_UP}
  Splitter1.ResizeStyle := rsUpdate;
{$ENDIF}
end;

end.
 